#include "../headers/DaemonNativeEventsCapturer.h"

using namespace DaemonNativeEvents;

Napi::FunctionReference DaemonNativeEventsCapturer::constructor;

Napi::Object DaemonNativeEventsCapturer::Init(Napi::Env env, Napi::Object exports) {
    Napi::HandleScope scope(env);
    Napi::Function func = DefineClass(env, "DaemonNativeEventsCapturer",
        {
            InstanceMethod("startListeners", &DaemonNativeEventsCapturer::ExportedStartListeners),
            InstanceMethod("initKeyPressEvents", &DaemonNativeEventsCapturer::exported_init_key_press_events),
            InstanceMethod("deInitKeyPressEvents", &DaemonNativeEventsCapturer::exported_deinit_key_press_events),
            InstanceMethod("initSdLoginEvents", &DaemonNativeEventsCapturer::exported_init_sd_login_events),
            InstanceMethod("deInitSdLoginEvents", &DaemonNativeEventsCapturer::exported_deinit_sd_login_events),
            InstanceMethod("getUsersLoginData", &DaemonNativeEventsCapturer::ExportedGetUsersStateLoginData),
            InstanceMethod("closeListeners", &DaemonNativeEventsCapturer::ExportedcloseListeners)
        }
    );

    constructor.SuppressDestruct();
    constructor = Napi::Persistent(func);

    exports.Set("DaemonNativeEventsCapturer", func);
    return exports;
}

DaemonNativeEventsCapturer::DaemonNativeEventsCapturer(const Napi::CallbackInfo &info) : Napi::ObjectWrap<DaemonNativeEventsCapturer>(info) {
    Napi::Env env = info.Env();
    Napi::HandleScope scope(env);

    // setting default value to all listener flags
    this->isThreadCreated = false;
    this->_loopCounter = 0;

    this->_isListenKeyPressTrackerEvents = false;
    this->_isListenSdLoginMonitorEvents = false;
}

/**
 * This function is exported to JS. It will be called only at start.
 * This launches the thread and initialises tsfn.
 */
void DaemonNativeEventsCapturer::ExportedStartListeners(const Napi::CallbackInfo &info) {
    Napi::Env env = info.Env();
    try {
        if (info.Length() == 1 && info[0].IsFunction()) {
            cout << "Arguments are correct. Creating TSFN Thread." << std::endl;
        } else {
            throw TypeError::New(env, "Expects exactly 1 Argument of type Function.");
        }

    } catch (const char *message) {
        Napi::Error::New(info.Env(), message).ThrowAsJavaScriptException();
        return;
    } catch (std::string message) {
        Napi::Error::New(info.Env(), message).ThrowAsJavaScriptException();
        return;
    }

    if (!this->isThreadCreated) {
        tsfn = ThreadSafeFunction::New(
            env,
            info[0].As<Function>(),        // JavaScript function called asynchronously
            "DaemonNativeEventsCapturer",  // Name
            0,                             // Unlimited queue
            1                              // Only one thread will use this initially
        );
        
        this->isThreadCreated = true;
        this->CreateNativeThread();
    } else {
        cout << "TSFN Thread is already created. So exiting from here." << std::endl;
    }
}

/**
 * This function is exported to JS. It is used to start listening to login events.
 * This can be initialized on calling startListener or can be initialized explicitly
 */
Napi::Value DaemonNativeEventsCapturer::exported_init_sd_login_events(const Napi::CallbackInfo &info) {
    Napi::Env env = info.Env();
    this->_isListenSdLoginMonitorEvents = this->loginEventsUtility.init();;
    return Napi::Boolean::New(env, this->_isListenSdLoginMonitorEvents);
}

/**
 * This function is exported to JS. It is used to start listening to key press events.
 * This can be initialized on calling startListener or can be initialized explicitly
 */
Napi::Value DaemonNativeEventsCapturer::exported_init_key_press_events(const Napi::CallbackInfo &info) {
    Napi::Env env = info.Env();
    this->_isListenKeyPressTrackerEvents = this->keyPressEventsTrackerUtility.init();
    return Napi::Boolean::New(env, this->_isListenKeyPressTrackerEvents);
}

/**
 * This function is exported to JS. This creates a thread safe function if not created.
 * On calling this, it starts listening to subscribed events. To listen to other events, need to call init function for respective events explicitly.
 */
void DaemonNativeEventsCapturer::CreateNativeThread() {
    this->nativeThread = std::thread([&] {
        while(this->isThreadCreated) {
            if (this->_loopCounter % GET_LOGIN_EVENTS_DETAILS_AFTER_X_TIMES == 0 && this->_isListenSdLoginMonitorEvents) {
                this->get_sd_user_details();
            }

            if (this->_isListenKeyPressTrackerEvents) {
                this->get_paste_event_details();
            }

            // If we are not listening to login events, then need to sleep explicitly
            std::this_thread::sleep_for(std::chrono::milliseconds(SLEEP_TIMER_FOR_LOOP));
            
            this->_loopCounter++;
            if (this->_loopCounter > RPC_MAX_LOOP_COUNTER) {
                this->_loopCounter = 1;
            }
        }

        if (napi_ok != tsfn.Release())
            cout << "Error on releasing thread-safe function\n";
        else
            cout << "Successfully released DaemonUtility Event thread-safe function\n";
    });
}

/**
 * Get sd user details, and emits a callback to JS
 */
void DaemonNativeEventsCapturer::get_sd_user_details() {
    auto userStateDataCallback = [this] (Napi::Env env, Function jsCallback, UserStateInfo* userStateInfo) {
        // jsCallBack needs two argument, count of arg, and all argument values.
        // Here we need 2 args. One event name, and two is napi_value for userStateInfo
        size_t argc = 2;
        napi_value argv[2];

        Napi::String eventName = Napi::String::New(env, LOGIN_USER_DATA.c_str());
        Napi::Array userStateInfoNapiArray = this->loginEventsUtility.convertCppToJavaScript(env, userStateInfo);

        Napi::Object userStateInfoObj = Napi::Object::New(env);
        userStateInfoObj.Set("userData", userStateInfoNapiArray);

        argv[0] = eventName;
        argv[1] = userStateInfoObj;
        jsCallback.Call(argc, argv);

        for(int i = 0; i < this->loginEventsUtility.getNumUid(); i++) {
            if (userStateInfo[i].numSessions)
                free(userStateInfo[i].userSessionInfo);
        }
        free(userStateInfo);
    };

    UserStateInfo* userStateInfo = this->loginEventsUtility.get_sd_user_details();
    if (userStateInfo != NULL) {
        napi_status status = tsfn.NonBlockingCall(userStateInfo, userStateDataCallback);
        if (napi_ok != status)
            cout << "User State event captured but got error at C++ in calling node event: " << status << endl;
    }
}

/**
 * Emits a callback to JS
 */
void DaemonNativeEventsCapturer::get_paste_event_details() {
    auto pasteEventCallback = [this] (Napi::Env env, Function jsCallback) {
        jsCallback.Call({String::New(env, PASTE_EVENT.c_str())});
    };

    if(this->keyPressEventsTrackerUtility.get_paste_event_details()) {
        napi_status status = tsfn.NonBlockingCall(pasteEventCallback);
        if (napi_ok != status)
            cout << "Ctrl+v event captured but got error at C++ in calling node event: " << status << endl;
    }
}

/**
 * This is an exported function to JS. It return the current active user state on request.
 * 
 */
Napi::Value DaemonNativeEventsCapturer::ExportedGetUsersStateLoginData(const Napi::CallbackInfo& info) {
    Napi::Env env = info.Env();
    return this->loginEventsUtility.getUsersStateLoginData(env);
}

/**
 * This is an exported function to JS. This is used to stop listening to sd login events
 */
void DaemonNativeEventsCapturer::exported_deinit_sd_login_events(const Napi::CallbackInfo& info) {
    if (this->_isListenSdLoginMonitorEvents) {
        this->_isListenSdLoginMonitorEvents = false;
        this->loginEventsUtility.deinit();
    }
}

/**
 * This is an exported function to JS. This is used to stop listening to sd key press events
 */
void DaemonNativeEventsCapturer::exported_deinit_key_press_events(const Napi::CallbackInfo& info) {
    if (this->_isListenKeyPressTrackerEvents) {
        this->_isListenKeyPressTrackerEvents = false;
        this->keyPressEventsTrackerUtility.deinit();
    }
}

/**
 * This is an exported function to JS. This is used to stop listening to all events. It will exit the thread.
 * If this is called, then need to call init and StartListener to start listening to required events again.
 */
void DaemonNativeEventsCapturer::ExportedcloseListeners(const Napi::CallbackInfo &info)
{
    cout << "DaemonNativeEventsCapturer::closeListener\n";
    this->isThreadCreated = false;
    this->exported_deinit_sd_login_events(info);
    this->exported_deinit_key_press_events(info);

    nativeThread.join();
}

DaemonNativeEventsCapturer::~DaemonNativeEventsCapturer(){
}

Napi::Object Init(Napi::Env env, Napi::Object exports){
    return DaemonNativeEventsCapturer::Init(env, exports);
}

NODE_API_MODULE(daemon_native_utility, Init);
