#include "../headers/LoginStateEvents.h"

#define GET_STATE_INFO(func, attr, holder)  \
{                                           \
    char *attr = NULL;                      \
    if (func(s, &attr) >= 0)                \
    {                                       \
        strcpy(holder.attr, attr);          \
        free (attr);                        \
    }                                       \
    else                                    \
        strcpy(holder.attr, "");            \
}

using namespace LoginEvents;

LoginStateEvents::LoginStateEvents() {
    // setting default values to user state members
    this->_uidNum = 0;
    this->_poll_fd[0].fd = 0;
    this->_poll_fd[0].events = (short int)0;
    this->_monitor = NULL;
}

/**
 * This function is exported to JS. It is used to start listening to login events.
 * This can be initialized on calling startListener or can be initialized explicitly
 */
// Napi::Value LoginStateEvents::exported_init_sd_login_events(const Napi::CallbackInfo &info) {
bool LoginStateEvents::init() {
    int ret = sd_login_monitor_new("uid", &_monitor);
    if (ret < 0)
    {
        cout << "new monitor object could not be created, " << strerror(-ret) << endl;
        sd_login_monitor_unref(_monitor);
        return false;
    }

    this->_poll_fd[0].fd = sd_login_monitor_get_fd(_monitor);
    this->_poll_fd[0].events = sd_login_monitor_get_events(_monitor);

    return true;
}

int LoginStateEvents::getNumUid() {
    return this->_uidNum;
}

/**
 * This is an exported function to JS. It return the current active user state on request.
 * 
 */

Napi::Value LoginStateEvents::getUsersStateLoginData(Napi::Env env) {
    Napi::Object userStateInfoObj = Napi::Object::New(env);

    uid_t *pUid = NULL;
    int nUid = sd_get_uids(&pUid);
    if (nUid != ENOMEM) {
        UserStateInfo* userStateInfo = this->getUserStateInfo(pUid, nUid);
        Napi::Array userStateInfoNapiArray = this->convertCppToJavaScript(env, userStateInfo);

        userStateInfoObj.Set("userData", userStateInfoNapiArray);

        for(int i = 0; i < nUid; i++) {
            if (userStateInfo[i].numSessions)
                free(userStateInfo[i].userSessionInfo);
        }
        free(userStateInfo);
        free(pUid);
    }

    Napi::Value userStateInfoNapiValue = Napi::Value(env, userStateInfoObj);
    return userStateInfoNapiValue;
}

/**
 * Get sd user details, and emits a callback to JS
 */
UserStateInfo* LoginStateEvents::get_sd_user_details() {
    int ret = 0;
    // poll() returns -ve on error, zero on timeout, and +ve if any requested event happens
    if (!ret) {
        ret = poll(this->_poll_fd, 1, 0);

        if (ret < 0) {
            cout << "poll timeout error " << strerror(errno) << endl;
            ret = 0;
        }
    }

    UserStateInfo* userStateInfo = NULL;
    if ((this->_poll_fd[0].revents & POLLERR) || (this->_poll_fd[0].revents & POLLHUP) || (this->_poll_fd[0].revents & POLLNVAL)) {
        cout << "revents returned some error " << this->_poll_fd[0].revents << endl;
    } else {
        uid_t *pUid = NULL;
        _uidNum = sd_get_uids(&pUid);
        if (_uidNum != ENOMEM) {
            // Save the message to the heap. The JavaScript marshaller (CallJs) will free this item after having sent it to JavaScript.
            userStateInfo = getUserStateInfo(pUid);
            free(pUid);
        }

        sd_login_monitor_flush(_monitor);
    }
    return userStateInfo;
}

/**
 * This function get user state info from the pUid and uidNum provided and return the same
 */
UserStateInfo* LoginStateEvents::getUserStateInfo(uid_t* pUid, int uidNum) {
    int uidNumToBeUsed = uidNum == -1 ? _uidNum : uidNum;
    _uidNum = uidNumToBeUsed;
    UserStateInfo* userStateInfo = (UserStateInfo*) malloc(sizeof(UserStateInfo) * uidNumToBeUsed);
    for(int i = 0; i < uidNumToBeUsed; i++)
    {
        userStateInfo[i].uid = pUid[i];
        char *state = NULL;
        if (sd_uid_get_state(pUid[i], &state) >= 0)
        {
            strcpy(userStateInfo[i].state, state);
            free (state);
        }
        else
            strcpy(userStateInfo[i].state, "state not found");
        // cout << "state of user " << userStateInfo[i].uid << " is: " << userStateInfo[i].state << endl;

        char **sessions = NULL;
        int numSessions = sd_uid_get_sessions(pUid[i], 0, &sessions);       // 0 will give all type of sessions
        // cout << "number of sessions: " << numSessions << endl;
        userStateInfo[i].numSessions = numSessions;

        if (numSessions > 0)
        {
            userStateInfo[i].userSessionInfo = (UserSessionInfo*) malloc(sizeof(UserSessionInfo)*numSessions);
            for (int k=0; k<numSessions; k++)
            {
                const char *s = sessions[k];
                UserSessionInfo userSessionInfo;
                strcpy(userSessionInfo.session, s);

                GET_STATE_INFO(sd_session_get_tty,     tty,       userSessionInfo)
                GET_STATE_INFO(sd_session_get_seat,    seat,      userSessionInfo)
                GET_STATE_INFO(sd_session_get_service, service,   userSessionInfo)
                GET_STATE_INFO(sd_session_get_type,    type,      userSessionInfo)
                GET_STATE_INFO(sd_session_get_class,   sessClass, userSessionInfo)
                GET_STATE_INFO(sd_session_get_state,   sessState, userSessionInfo)

                // cout << "uid " << pUid[i] << ",type " << userSessionInfo.type << ",tty " << userSessionInfo.tty
                //      << ",seat " << userSessionInfo.seat << ",service " << userSessionInfo.service << ",class " << userSessionInfo.sessClass
                //      << ",session state " << userSessionInfo.sessState << ",session " << sessions[k] << endl;

                userStateInfo[i].userSessionInfo[k] = userSessionInfo;
                free(sessions[k]);
            }
            free(sessions);
        }
    }
    return userStateInfo;
}

/**
 * This function return Napi::Array equivalent of UserStateInfo struct object
 */
Napi::Array LoginStateEvents::convertCppToJavaScript(Napi::Env env, UserStateInfo *userStateInfo, int n) {
    int nUidToBeUsed = n == -1 ? _uidNum : n;
    Napi::Array finalArray = Napi::Array::New(env, nUidToBeUsed);

    for(int i = 0; i < nUidToBeUsed; i++) {
        Napi::Object userObj = Napi::Object::New(env);
        Napi::Value uid = Napi::Number::New(env, userStateInfo[i].uid);
        Napi::String state = Napi::String::New(env, userStateInfo[i].state);

        userObj.Set("uid", uid);
        userObj.Set("state", state);

        if (userStateInfo[i].numSessions) {
            Napi::Array sessionArray = Napi::Array::New(env, userStateInfo[i].numSessions);

            for(int k = 0; k < userStateInfo[i].numSessions; k++) {
                UserSessionInfo userSessionInfo = userStateInfo[i].userSessionInfo[k];

                Napi::Object sessionObj = Napi::Object::New(env);
                sessionObj.Set("session", String::New(env, userSessionInfo.session));
                sessionObj.Set("tty", String::New(env, userSessionInfo.tty));
                sessionObj.Set("seat", String::New(env, userSessionInfo.seat));
                sessionObj.Set("service", String::New(env, userSessionInfo.service));
                sessionObj.Set("type", String::New(env, userSessionInfo.type));
                sessionObj.Set("sessClass", String::New(env, userSessionInfo.sessClass));
                sessionObj.Set("sessState", String::New(env, userSessionInfo.sessState));

                sessionArray[k] = sessionObj;
            }
            userObj.Set("userSessionInfo", sessionArray);
        }
        finalArray[i] = userObj;
    }
    return finalArray;
}

void LoginStateEvents::deinit() {
    sd_login_monitor_unref(_monitor);
    sd_login_monitor_flush(_monitor);
    this->_monitor = NULL;
}

LoginStateEvents::~LoginStateEvents()
{
    free(this->_monitor);
}
