export declare const displayUtility: INativeDisplayUtility;
export declare const screenCaptureUtility: IScreenCaptureUtility;
export declare const lockUtility: ILockUtility;
export declare const ttyKeyboardUtility: ITTYKeyboardUtility;
export declare const xKeyboardModule: IXKeyboardModule;
/**
 * @export
 * @extends {IResolution}
 * height - height component of the resolution
 * width - width component of the resolution
 */
export interface IResolution {
    width: number;
    height: number;
}
/**
 * @export
 * @extends {IResolution}
 * offsetX - x offset of the output (monitor)
 * offsetY - y offset of the output (monitor)
 * rrOutput - rr output of the output (monitor)
 */
export interface IResolutionWithOffset extends IResolution {
    offsetX: number;
    offsetY: number;
    rrOutput: number;
}
interface INativeDisplayUtility {
    /**
     * Returns list of numbers which represents the RROutput corresponding to all the outputs
     * undefined returned in case of any error or no connected outputs
     */
    getConnectedOutputs(): number[] | undefined;
    /**
     * Returns the name of the output (monitor) given the RROutput number for that output
     */
    getOutputName(rROutput: number): string;
    /**
     * Returns the current resolution of the output (monitor) given the RROutput number for that output
     */
    getCurrentResolution(rROutput: number): IResolution | undefined;
    /**
     * Returns the list of possible resolutions for an output (montior) given the RROutput number for that output
     */
    getResolutions(rROutput: number): IResolution[] | undefined;
    /**
     * Sets a particular resolution for an output (monitor) given the RROutput number for that output and the new resolution to be set.
     */
    setResolution(rROutput: number, resolution: IResolution): boolean;
    /**
     * Reduces the brightness of all connected outputs to 0 and makes the screen blank.
     */
    makeScreenBlank(): void;
    /**
     * Reverses the operation performed by makeScreenBlank()
     */
    reverseBlankScreen(): void;
    /**
     * Get the RROutput number corresponding to the primary monitor
     * Note: When no monitor is marked as primary, one of the connected monitor's RROutput number is returned
     */
    getPrimaryRROutput(): number;
    /**
     * Get the complete resolution of the display when multiple monitors are connected.
     * Note: Returns same value as getCurrentResolution(), when single monitor is connected
     */
    getExtendedMonitorResolution(): IResolution | undefined;
    /**
     * Get the position (x-offset and y-offset), height and width of connected monitors
     */
    getAllCurrentResolutionsWithOffset(): IResolutionWithOffset[] | undefined;
    /**
     * Returns true if the display is accessible else false
     */
    isDisplayAvailable(): boolean;
    /**
     * sets up the outputs for split-viewer, and returns the old position of ouputs and new position of outputs as per the new alignment
     */
    setupToEnableSplitViewer(): string;
    /**
     * Receives old position of outputs and restores the positions of outputs as they were before split-viewer
     */
    setupToDisableSplitViewer(coordinateString: string): void;
}
interface ILockUtility {
    /**
     * Locks the linux computer when executed
     */
    lockScreen(): void;
}
interface IScreenCaptureUtility {
    /**
     * Sets the application name: remotepc, remotepc-host, remotepc-cli
     */
    setAppName(appName: string): void;
    /**
     * Initialises the screen capturer and x264 encoder for capturing screen
     */
    init(singleMonitorCapture: boolean, rROutput?: number): void;
    /**
     * Returns the captured and encoded frame buffer as a callback
     */
    getNextFrame(callback: (nextFrame: ArrayBuffer) => void): void;
    /**
     * Returns the captured and encoded frame buffer as a callback
     * Returns only if there is a change compared to previous captured frame.
     * In case of no changes, every 30ms we look for frame change and return next frame
     * If getIFrame is passed as true, next frame will be generated as IFrame
     */
    /**
     * Force callback from getNextFrame function if it is waiting for a change in frame
     */
    forceNextFrame(): void;
    /**
     * Force the next frame to be an iframe. Call will also force the next frame and return any waiting getNextFrame callback
     */
    sendNextFrameAsIFrame(): void;
    /**
     * CRF refers to Constant Rate Factor parameter that is used to modify quality of encoding.
     * This function can be used to reducing encoding quality in order to get smaller frames for slow networks.
     * @param crfValue CRF value to be set
     */
    setCRFValue(crfValue: number): void;
    /**
     * Returns the resolution of framebuffer in pixels
     */
    getFrameBufferResolution(): string;
    /**
     * Set Show Remote Cursor Flag. If true, then cursor will be captured alongwith screen
     */
    setShowRemoteCursorFlag(showCursorFlag: boolean): void;
}
interface ITTYKeyboardUtility {
    /**
     * Creates the fake keyboard
     */
    initialiseTtyInput(): void;
    /**
     * Destroys the fake keyboard
     */
    deinitialiseTtyInput(): void;
    /**
     * simulates key presses and key releases
     */
    processTtyInput(keyCode: number, isPressed: number): void;
}
interface IXKeyboardModule {
    init(): void;
    reInitializeInfo(): boolean;
    groupCount(): number;
    currentGroupNum(): number;
    currentGroupName(): IKeyboardLayoutName;
    currentGroupVariant(): IKeyboardLayoutName;
    currentGroupSymbol(): IKeyboardLayoutName;
    changeGroup(groupNumber: number): boolean;
    getAllGroupSymbol(): IKeyboardLayoutArray;
    /**
     * Invoke key down and key up of a unicode character
     * @param unicodeHexValue Starts with U followed by octal value of the unicode character. Example : U0633
     */
    unicodeTap(unicodeHexValue: string): void;
}
interface IKeyboardLayoutName {
    name: string;
}
interface IKeyboardLayoutArray {
    layouts: string[];
}
export {};
//# sourceMappingURL=index.d.ts.map