"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FileSystemExplorer = void 0;
const child_process_1 = require("child_process");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const constants_1 = require("./constants");
class FileSystemExplorer {
    constructor() {
    }
    createFileSystemTree(rootPathToDir, options, onEachFile, onEachDirectory) {
        if (options !== undefined && options.maxNumberOfFiles !== undefined) {
            if (options.maxNumberOfFiles === 0) {
                return;
            }
            else {
                options.maxNumberOfFiles -= 1;
            }
        }
        let localPathToDir = rootPathToDir;
        const name = path_1.default.basename(localPathToDir);
        localPathToDir = options !== undefined && options.normalizePath !== undefined && options.normalizePath ?
            this.normalizePath(localPathToDir) : localPathToDir;
        let rootPathInfo = {
            path: localPathToDir,
            name,
            size: 0,
            mtime: new Date(),
            ctime: new Date(),
            atime: new Date(),
            type: constants_1.FILE_TYPE.DIRECTORY
        };
        let rootPathStats;
        try {
            rootPathStats = fs_1.default.lstatSync(localPathToDir);
        }
        catch (e) {
            console.warn(`Unable to find stats for the given path: ${localPathToDir}`);
            return undefined;
        }
        if (options !== undefined && options.exclude !== undefined) {
            const excludes = this.isRegEx(options.exclude) ? [options.exclude] : options.exclude;
            if (excludes.some((exclusion) => exclusion.test(localPathToDir))) {
                return undefined;
            }
        }
        if (rootPathStats.isFile()) {
            rootPathInfo = this.getFileDetails(localPathToDir, rootPathInfo, rootPathStats, onEachFile, constants_1.FILE_TYPE.FILE);
        }
        else if (rootPathStats.isSymbolicLink()) {
            if (options !== undefined && options.resolveSymbolicLinks !== undefined && options.resolveSymbolicLinks) {
                try {
                    const isDirectory = fs_1.default.lstatSync(fs_1.default.realpathSync(localPathToDir))
                        .isDirectory();
                    if (isDirectory) {
                        rootPathInfo = this.getDirectoriesAndFiles(localPathToDir, rootPathInfo, rootPathStats, options, onEachFile, onEachDirectory);
                    }
                    else {
                        rootPathStats = fs_1.default.lstatSync(fs_1.default.realpathSync(localPathToDir));
                        rootPathInfo = this.getFileDetails(localPathToDir, rootPathInfo, rootPathStats, onEachFile, constants_1.FILE_TYPE.FILE);
                    }
                }
                catch (err) {
                    console.log(`lStatSync for '${(localPathToDir)}' failed. Skipping current file.`);
                    return undefined;
                }
            }
            else {
                rootPathInfo = this.getFileDetails(localPathToDir, rootPathInfo, rootPathStats, onEachFile, constants_1.FILE_TYPE.SYMB_LINK);
            }
        }
        else if (rootPathStats.isDirectory()) {
            rootPathInfo = this.getDirectoriesAndFiles(localPathToDir, rootPathInfo, rootPathStats, options, onEachFile, onEachDirectory);
        }
        else {
            return undefined;
        }
        return rootPathInfo;
    }
    getFileDetails(localPathToDir, rootPathInfo, rootPathStats, onEachFile, fileType = constants_1.FILE_TYPE.FILE) {
        const fileExtension = path_1.default.extname(localPathToDir)
            .toLowerCase();
        rootPathInfo.size = rootPathStats.size;
        rootPathInfo.extension = fileExtension;
        rootPathInfo.type = fileType;
        rootPathInfo.atime = rootPathStats.atime;
        rootPathInfo.mtime = rootPathStats.mtime;
        rootPathInfo.ctime = rootPathStats.ctime;
        if (onEachFile !== undefined) {
            onEachFile(rootPathInfo, localPathToDir, rootPathStats);
        }
        return rootPathInfo;
    }
    getDirectoriesAndFiles(localPathToDir, rootPathInfo, rootPathStats, options, onEachFile, onEachDirectory) {
        try {
            const dirContent = this.safeReadDirSync(localPathToDir);
            if (dirContent === undefined) {
                return undefined;
            }
            rootPathInfo.children = dirContent.map((child) => {
                return this.createFileSystemTree(path_1.default.join(localPathToDir, child), options, onEachFile, onEachDirectory);
            })
                .filter((e) => {
                return (e !== undefined);
            });
            rootPathInfo.size = rootPathInfo.children.reduce((prev, cur) => prev + cur.size, 0);
            rootPathInfo.childCount = rootPathInfo.children.length;
            rootPathInfo.type = constants_1.FILE_TYPE.DIRECTORY;
            rootPathInfo.atime = rootPathStats.atime;
            rootPathInfo.mtime = rootPathStats.mtime;
            rootPathInfo.ctime = rootPathStats.ctime;
            if (onEachDirectory !== undefined) {
                onEachDirectory(rootPathInfo, localPathToDir, rootPathStats);
            }
            return rootPathInfo;
        }
        catch (err) {
            console.log(`getDirectoriesAndFiles failed. Error: ${err}`);
            return undefined;
        }
    }
    getDateFormatMMDDYYYY(date) {
        return `${date.getMonth() + 1}/${date.getDate()}/${date.getFullYear()} \
${date.getHours()}:${date.getMinutes()}:${date.getSeconds()}`;
    }
    getAllDirectoryPathFromFileSystem(rootPathToDir, options, onEachFile, onEachDirectory) {
        const fileSystemListInfo = this.createFileSystemTree(rootPathToDir, options, onEachFile, onEachDirectory);
        if (fileSystemListInfo === undefined) {
            return [];
        }
        return this.getAllDirectoryPathFromPreparedFSInfo(fileSystemListInfo);
    }
    getAllDirectoryPathFromPreparedFSInfo(fileSystemListInfo) {
        const allDirectoriesPath = [];
        let directoryCount = 0;
        const traverseFileSystemDFS = (fsListInfo) => {
            var _a;
            if (fsListInfo.type === constants_1.FILE_TYPE.DIRECTORY) {
                allDirectoriesPath.push(fsListInfo.path);
                directoryCount += 1;
            }
            (_a = fsListInfo.children) === null || _a === void 0 ? void 0 : _a.forEach((child) => {
                traverseFileSystemDFS(child);
            });
        };
        traverseFileSystemDFS(fileSystemListInfo);
        console.info(`Total Directories Count from Path "${fileSystemListInfo.path}": ${directoryCount}`);
        return allDirectoriesPath;
    }
    getAllFilePathFromFileSystem(rootPathToDir, options, onEachFile, onEachDirectory) {
        const fileSystemListInfo = this.createFileSystemTree(rootPathToDir, options, onEachFile, onEachDirectory);
        if (fileSystemListInfo === undefined) {
            return [];
        }
        return this.getAllFilePathFromPreparedFSInfo(fileSystemListInfo);
    }
    getAllFilePathFromPreparedFSInfo(fileSystemListInfo) {
        const allFilesPath = [];
        let filesCount = 0;
        const traverseFileSystemDFS = (fsListInfo) => {
            var _a;
            if (fsListInfo.type === constants_1.FILE_TYPE.FILE) {
                allFilesPath.push(fsListInfo.path);
                filesCount += 1;
            }
            (_a = fsListInfo.children) === null || _a === void 0 ? void 0 : _a.forEach((child) => {
                traverseFileSystemDFS(child);
            });
        };
        traverseFileSystemDFS(fileSystemListInfo);
        return allFilesPath;
    }
    getAllFSElementsDetailsFromFileSystem(rootPathToDir, options, onEachFile, onEachDirectory) {
        const fileSystemListInfo = this.createFileSystemTree(rootPathToDir, options, onEachFile, onEachDirectory);
        if (fileSystemListInfo === undefined) {
            return [];
        }
        return (this.getAllFSElementsDetailsFromPreparedFSInfo(fileSystemListInfo));
    }
    getAllFSElementsDetailsFromPreparedFSInfo(fileSystemListInfo) {
        const allFSElementsDetails = [];
        let eachFSElement = {
            type: constants_1.FILE_TYPE.FILE,
            size: 0,
            name: '',
            mtime: '',
            path: ''
        };
        const traverseFileSystemDFS = (fsListInfo) => {
            var _a;
            eachFSElement = {
                type: fsListInfo.type,
                size: fsListInfo.size,
                name: fsListInfo.name,
                mtime: this.getDateFormatMMDDYYYY(fsListInfo.mtime),
                path: fsListInfo.path,
                extension: fsListInfo.extension,
                childCount: fsListInfo.childCount
            };
            allFSElementsDetails.push(eachFSElement);
            (_a = fsListInfo.children) === null || _a === void 0 ? void 0 : _a.forEach((child) => {
                traverseFileSystemDFS(child);
            });
        };
        traverseFileSystemDFS(fileSystemListInfo);
        return allFSElementsDetails;
    }
    getDirectoryContentList(pathToDir, options, onEachDirContent) {
        let dirContentList = [];
        let localPathToDir = pathToDir;
        try {
            const dirContentNames = this.safeReadDirSync(pathToDir);
            if (dirContentNames === undefined) {
                return [];
            }
            localPathToDir = options !== undefined && options.normalizePath !== undefined && options.normalizePath ?
                this.normalizePath(localPathToDir) : localPathToDir;
            if (options !== undefined && options.exclude !== undefined) {
                const excludes = this.isRegEx(options.exclude) ? [options.exclude] : options.exclude;
                if (excludes.some((exclusion) => exclusion.test(localPathToDir))) {
                    return [];
                }
            }
            let dirFileInfo;
            dirContentList = dirContentNames.map((eachDirFileName) => {
                dirFileInfo = {
                    name: eachDirFileName,
                    size: 0,
                    mtime: this.getDateFormatMMDDYYYY(new Date()),
                    type: constants_1.FILE_TYPE.FILE
                };
                let eachDirFileStats;
                try {
                    eachDirFileStats = fs_1.default.lstatSync(path_1.default.join(localPathToDir, eachDirFileName));
                }
                catch (e) {
                    console.warn(`Unable to find stats for the given path: ${localPathToDir}`);
                    return undefined;
                }
                if (!isNaN(eachDirFileStats.mtime.getFullYear())) {
                    dirFileInfo.mtime = this.getDateFormatMMDDYYYY(eachDirFileStats.mtime);
                }
                else {
                    const pathToUse = path_1.default.join(localPathToDir.replace(/[\\&"')( ]/g, '\\$&'), eachDirFileName.replace(/[\\&"')( ]/g, '\\$&'));
                    dirFileInfo.mtime = this.checkForPathAndFindTime(pathToUse);
                }
                dirFileInfo.size = eachDirFileStats.size;
                if (eachDirFileStats.isFile() || eachDirFileStats.isSymbolicLink()) {
                    const fileExtension = path_1.default.extname(eachDirFileName)
                        .toLowerCase();
                    dirFileInfo.extension = fileExtension;
                    dirFileInfo.type = eachDirFileStats.isFile() ? constants_1.FILE_TYPE.FILE : constants_1.FILE_TYPE.SYMB_LINK;
                }
                else if (eachDirFileStats.isDirectory()) {
                    dirFileInfo.type = constants_1.FILE_TYPE.DIRECTORY;
                }
                else {
                    return undefined;
                }
                if (onEachDirContent !== undefined) {
                    onEachDirContent(path_1.default.join(localPathToDir, eachDirFileName), eachDirFileStats);
                }
                return dirFileInfo;
            })
                .filter((e) => {
                return (e !== undefined);
            });
            return dirContentList;
        }
        catch (err) {
            console.log(`getDirectoryContentList failed. Error: ${err}`);
            return [];
        }
    }
    getDirectoryContentListForJSON(pathToDir, options, onEachDirContent) {
        let dirContentList = [];
        let localPathToDir = pathToDir;
        try {
            const dirContentNames = this.safeReadDirSync(pathToDir);
            if (dirContentNames === undefined) {
                return [];
            }
            localPathToDir = options !== undefined && options.normalizePath !== undefined && options.normalizePath ?
                this.normalizePath(localPathToDir) : localPathToDir;
            if (options !== undefined && options.exclude !== undefined) {
                const excludes = this.isRegEx(options.exclude) ? [options.exclude] : options.exclude;
                if (excludes.some((exclusion) => exclusion.test(localPathToDir))) {
                    return [];
                }
            }
            let dirFileInfo;
            dirContentList = dirContentNames.map((eachDirFileName) => {
                dirFileInfo = {
                    m_name: eachDirFileName,
                    lsize: '0',
                    m_lmd: this.getDateFormatMMDDYYYY(new Date()),
                    nType: constants_1.FILE_TYPE.FILE,
                    m_FullPath: path_1.default.join(pathToDir, eachDirFileName)
                };
                let eachDirFileStats;
                try {
                    eachDirFileStats = fs_1.default.lstatSync(path_1.default.join(localPathToDir, eachDirFileName));
                }
                catch (e) {
                    console.warn(`Unable to find stats for the given path: ${localPathToDir}`);
                    return undefined;
                }
                if (!isNaN(eachDirFileStats.mtime.getFullYear())) {
                    dirFileInfo.m_lmd = this.getDateFormatMMDDYYYY(eachDirFileStats.mtime);
                }
                else {
                    const pathToUse = path_1.default.join(localPathToDir.replace(/[\\&"')( ]/g, '\\$&'), eachDirFileName.replace(/[\\&"')( ]/g, '\\$&'));
                    dirFileInfo.m_lmd = this.checkForPathAndFindTime(pathToUse);
                }
                dirFileInfo.lsize = eachDirFileStats.size.toString();
                if (eachDirFileStats.isFile() || eachDirFileStats.isSymbolicLink()) {
                    const fileExtension = path_1.default.extname(eachDirFileName)
                        .toLowerCase();
                    dirFileInfo.extension = fileExtension;
                    dirFileInfo.nType = eachDirFileStats.isFile() ? constants_1.FILE_TYPE.FILE : constants_1.FILE_TYPE.SYMB_LINK;
                }
                else if (eachDirFileStats.isDirectory()) {
                    dirFileInfo.nType = constants_1.FILE_TYPE.DIRECTORY;
                }
                else {
                    return undefined;
                }
                if (onEachDirContent !== undefined) {
                    onEachDirContent(path_1.default.join(localPathToDir, eachDirFileName), eachDirFileStats);
                }
                return dirFileInfo;
            })
                .filter((e) => {
                return (e !== undefined);
            });
            return dirContentList;
        }
        catch (err) {
            console.log(`getDirectoryContentList failed. Error: ${err}`);
            return [];
        }
    }
    checkForPathAndFindTime(pathToUse) {
        try {
            let dateTimeArray = [];
            let dateFormat = [];
            let timeFormat = [];
            const dateBefore1970 = (child_process_1.execSync(`stat -c %y ${pathToUse}`))
                .toString()
                .trim();
            dateTimeArray = dateBefore1970.split(' ');
            dateFormat = dateTimeArray[0].split('-');
            timeFormat = dateTimeArray[1].split('.');
            return `${dateFormat[1]}/${dateFormat[2]}/${dateFormat[0]} ${timeFormat[0]}`;
        }
        catch (err) {
            console.error(`getDirectoryDate() => ${err}`);
            return 'Invalid Date';
        }
    }
    createDirectory(destinationPath, isNewDirectoryNameIncluded = false, isRecursive = false) {
        const mkdirResponse = {
            errno: constants_1.FS_OK,
            completePath: '',
            fileName: ''
        };
        try {
            const newDirectoryName = isNewDirectoryNameIncluded ? path_1.default.basename(destinationPath) : 'New Folder';
            const destinationPathWithoutDirname = isNewDirectoryNameIncluded ?
                destinationPath.slice(0, destinationPath.lastIndexOf(newDirectoryName)) : destinationPath;
            let newFile = newDirectoryName;
            if (isNewDirectoryNameIncluded) {
                console.log(`Path: ${destinationPathWithoutDirname}, Dirname: ${newDirectoryName}`);
                fs_1.default.mkdirSync(path_1.default.join(destinationPathWithoutDirname, newDirectoryName), { recursive: isRecursive });
            }
            else {
                let i;
                if (fs_1.default.existsSync(path_1.default.join(destinationPathWithoutDirname, newFile))) {
                    console.info(`Directory name 'New Folder' already exists in ${newFile}. Appending number.`);
                    i = 1;
                    newFile = `${newDirectoryName} ${(i)}`;
                    while (fs_1.default.existsSync(path_1.default.join(destinationPathWithoutDirname, newFile))) {
                        i = i + 1;
                        newFile = `${newDirectoryName} ${(i)}`;
                    }
                }
                fs_1.default.mkdirSync(path_1.default.join(destinationPathWithoutDirname, newFile));
            }
            mkdirResponse.errno = constants_1.FS_OK;
            mkdirResponse.completePath = path_1.default.join(destinationPathWithoutDirname, newFile);
            mkdirResponse.fileName = newFile;
            return mkdirResponse;
        }
        catch (err) {
            console.error(`Create Directory inside '${destinationPath}' failed. Errno: ${err.errno} ${err.message}`);
            mkdirResponse.errno = err.errno !== undefined ? err.errno : constants_1.FS_ERR;
            mkdirResponse.completePath = err.message;
            return mkdirResponse;
        }
    }
    deleteDirectory(destinationPath, isDirectory = false) {
        const rmdirResponse = {
            errno: constants_1.FS_OK,
            completePath: '',
            fileName: ''
        };
        try {
            console.debug(`Checking if ${destinationPath} exists or not`);
            if (fs_1.default.existsSync(destinationPath)) {
                if (isDirectory) {
                    console.info(`Deleting Directory tree '${destinationPath}'`);
                    fs_1.default.rmdirSync(destinationPath, { recursive: true });
                }
                else {
                    console.info(`Deleting File '${destinationPath}'`);
                    fs_1.default.unlinkSync(destinationPath);
                }
            }
            return rmdirResponse;
        }
        catch (err) {
            console.error(`Deleting Directory '${destinationPath}' failed. Errno: ${err.errno} ${err.message}`);
            rmdirResponse.errno = err.errno !== undefined ? err.errno : constants_1.FS_ERR;
            rmdirResponse.completePath = err.message;
            return rmdirResponse;
        }
    }
    renameFile(oldPath, newPath) {
        const renameFileResp = {
            errno: constants_1.FS_OK,
            completePath: '',
            fileName: ''
        };
        try {
            console.debug(`check if destination path is available or not`);
            if (fs_1.default.existsSync(newPath)) {
                renameFileResp.errno = 13;
                renameFileResp.completePath = 'Destination Path already exists';
                return renameFileResp;
            }
            if (fs_1.default.existsSync(oldPath)) {
                console.info(`Renaming "${oldPath}" to "${newPath}"`);
                fs_1.default.renameSync(oldPath, newPath);
            }
            return renameFileResp;
        }
        catch (err) {
            console.error(`Rename File Failed: Errno: ${err.errno}. ${err.message}`);
            renameFileResp.errno = err.errno !== undefined ? err.errno : constants_1.FS_ERR;
            renameFileResp.completePath = err.message;
            return renameFileResp;
        }
    }
    checkDirectoryReadWritePersmission(pathToDir, mode = fs_1.default.constants.W_OK) {
        try {
            fs_1.default.accessSync(pathToDir, mode);
            console.info(`${pathToDir} is accessible for mode: ${mode}`);
            return constants_1.FS_OK;
        }
        catch (err) {
            if (err.code === 'EACCES' || err.code === 'EPERM') {
                console.warn(`User doesn't have access to "${pathToDir}". Ignoring Directory`);
            }
            else {
                console.warn(`Unable to Read from given path "${pathToDir}". Errno: ${err.errno}. Details: ${err.message}`);
            }
            return err.errno;
        }
    }
    checkFSElementExists(pathFSElement) {
        try {
            if (fs_1.default.existsSync(pathFSElement)) {
                return true;
            }
            else {
                return false;
            }
        }
        catch (err) {
            if (err.code === 'EACCES' || err.code === 'EPERM') {
                console.warn(`User doesn't have access to "${pathFSElement}". Ignoring Directory`);
            }
            else {
                console.warn(`Unable to Read from given path "${pathFSElement}". Errno: ${err.errno}. Details: ${err.message}`);
            }
            return false;
        }
    }
    getFSElementDetails(fsElementPath) {
        var _a;
        const fsElementDetails = {
            name: path_1.default.basename(fsElementPath),
            path: fsElementPath,
            type: constants_1.FILE_TYPE.FILE,
            mtime: this.getDateFormatMMDDYYYY(new Date()),
            size: 0
        };
        try {
            const fsElementStats = fs_1.default.lstatSync(fsElementPath);
            fsElementDetails.type = fsElementStats.isDirectory() ? constants_1.FILE_TYPE.DIRECTORY :
                (fsElementStats.isFile() ? constants_1.FILE_TYPE.FILE : constants_1.FILE_TYPE.SYMB_LINK);
            if (!isNaN(fsElementStats.mtime.getFullYear())) {
                fsElementDetails.mtime = this.getDateFormatMMDDYYYY(fsElementStats.mtime);
            }
            else {
                fsElementDetails.mtime = this.checkForPathAndFindTime((_a = fsElementDetails.path) === null || _a === void 0 ? void 0 : _a.replace(/[\\&"')( ]/g, '\\$&'));
            }
            fsElementDetails.size = fsElementStats.size;
            fsElementDetails.extension = fsElementDetails.type === constants_1.FILE_TYPE.FILE ? path_1.default.extname(fsElementPath)
                .toLowerCase() : undefined;
            return fsElementDetails;
        }
        catch (e) {
            console.warn(`Unable to find stats for the given path: ${fsElementPath}`);
            return fsElementDetails;
        }
    }
    checkMountFreeSpace(pathToFSElement) {
        try {
            const mountNameForFSElement = (child_process_1.execSync(`df "${pathToFSElement}" | tail -1 | awk '{ print $6 }'`)).toString()
                .trim();
            return (child_process_1.execSync(`df -h ${mountNameForFSElement} --block-size=1024 | tail -1 | awk '{print $4}'`))
                .toString()
                .trim();
        }
        catch (err) {
            console.info(`Failed to execute command to get free space from mount. ${err.stack}`);
            return '0';
        }
    }
    normalizePath(absolutePath) {
        return absolutePath.replace(/\\/g, '/');
    }
    safeReadDirSync(absolutePath) {
        let dirContents = [];
        try {
            dirContents = fs_1.default.readdirSync(absolutePath);
        }
        catch (err) {
            if (err.code === 'EACCES' || err.code === 'EPERM') {
                console.warn(`User doesn't have access to "${absolutePath}". Ignoring Directory`);
            }
            else {
                console.warn(`Unable to Read from given path "${absolutePath}". Errno: ${err.errno}. Details: ${err.message}`);
            }
            return undefined;
        }
        return dirContents;
    }
    isRegEx(regExp) {
        return typeof regExp === 'object' && regExp.constructor === RegExp;
    }
}
exports.FileSystemExplorer = FileSystemExplorer;
