/**
 * DisplayUtility will retrieve the information about the desktop display.
 * ScreenCaptureUtility will provide functions to capture screen and encode it.
 * Both these utilities are exposed by single .node file display-utility.node
 */
export declare const addon: any;
export declare const quicP2PAddOn: any;
export declare const suiteNativeEventsCapturer: ISuiteNativeEventsCapturer;
export declare const quicPeer2PeerWrapper: IQuicP2PWrapper;
export interface ISuiteNativeEventsCapturer {
    /**
     * Created a thread for tsfn created and start listening to required events
     * @param func Takes a function (an emitter function) and defines a thread-safe function for C to pass events to node.js
     * It monitors user events like cursor change, screen res change, clipboard events, screen lock detect,
     * and places a non-blocking call of the tsfn created
     */
    startListeners(func: Function): void;
    /**
     * @returns bool indicating whether clipboard copy events can be captured or not
     */
    initClipboardChangeEvents(): boolean;
    /**
     * @returns bool indicating whether cursor change events can be captured or not
     */
    initCursorChangeEvents(): boolean;
    /**
     * @returns bool indicating whether screen lock signal message can be processed or not for given OS
     */
    initScreenLockChangeEvents(osType: number): boolean;
    /**
     * @returns bool indicating whether screen resolution change events can be captured or not
     */
    initScreenResolutionChangeEvents(): boolean;
    /**
     * stop listening to cursor change events
     */
    deInitCursorChangeEvents(): void;
    /**
     * stop listening to clipboard copy events
     */
    deInitClipboardChangeEvents(): void;
    /**
     * stop processing screen lock signal message
     */
    deInitScreenLockChangeEvents(): void;
    /**
     * stop processing screen resolution change message
     */
    deInitScreenResolutionChangeEvents(): void;
    /**
     * Stops listening to all the events and releases tsfn
     */
    closeListeners(): void;
}
export interface IQuicP2PWrapper {
    /**
     * creates a new signal for Peer2Peer connection. Internally it tries to get local pair info, which will be shared to other party.
     * Make sure to listen for QUIC_LOCAL_PAIR_INFO event after calling this.
     * As p2p_init and get_pair_info is blocking, it is written in thread to make it non-blocking.
     * Make sure to listen to QUIC_LOCAL_PAIR_INFO event, which gives pair info and signal handle as data
     * @param func emitter function
     * @param isHost whether a dialer or receiver
     * @param stunURL stun URL
     */
    p2pInit1(func: Function, isHost: boolean, stunURL: string): void;
    p2pInit2(isHost: boolean, stunURL: string): void;
    /**
     * p2p_connect is infinitely blocking, so it is called inside a thread.
     * @param signalHandle connection handle
     * @param remotePairInfo pair info received from other party
     * @param timeout timeout in seconds
     */
    p2pConnect1(signalHandle: number, remotePairInfo: string, timeout: number): void;
    p2pConnect2(signalHandle: number, remotePairInfo: string, timeout: number): void;
    /**
     * function which calls writes data to queue, which is monitored by another thread to sequential send
     * @param signalHandle signal handle for the connection
     * @param length length of message to be sent
     * @param data data to be sent
     */
    send1(signalHandle: number, length: number, data: ArrayBufferLike): void;
    send2(signalHandle: number, length: number, data: ArrayBufferLike): void;
    /**
     * function closes the p2p channel, and responsible for release of all memory, tsfn and joining the threads
     */
    p2pClose1(): number;
    p2pClose2(): number;
    /**
     * function release thread safe function. This has to be called on end session.
     */
    releaseTSFunc(): void;
}
export interface ICursorShapeDetails {
    x: number;
    y: number;
    name: string;
    xhot: number;
    yhot: number;
    width: number;
    height: number;
}
export interface ICursorShapeData extends ICursorShapeDetails {
    pixels: number[];
}
export interface IPeer2PeerMessage {
    data: ArrayBuffer;
    length: number;
    signalHandle: number;
}
export interface IQuicPairInfo {
    pInfoLength: number;
    lPairInfo: string;
    rPairInfo: string;
    signalValue: number;
    channelNumber: number;
}
export interface IIceStateMessage {
    signalHandle: number;
    iceState: number;
    channelNumber: number;
}
export declare enum ICE_CANDIDATE_STATE {
    NEW = 1,
    CHECKING = 2,
    CONNECTED = 3,
    COMPLETED = 4,
    FAILED = 5,
    DISCONNECTED = 6,
    CLOSED = 7,
    INVALID = 8,
    OTHER = 9
}
export declare enum QUIC_P2P_EVENTS {
    PRE_SWITCH_P2P = "pre-switch-p2p",
    SWITCH_TO_P2P = "switch-to-p2p",
    REINIT_ON_PINGPONG_FAIL = "reinit-on-pingpong-fail",
    QUIC_MESSAGE_RECEIVED = "quic-message-received",
    QUIC_LOCAL_PAIR_INFO = "quic-local-pair-info",
    ICE_STATE_CHANGE = "ice-state-change"
}
//# sourceMappingURL=index.d.ts.map